from argparse import ArgumentParser, Namespace

from clime_bus_factor.version import version

name: str = "PRiMe"
versionName: str = "PRiMe Bus Factor Module"
authors: list = [
    "Nicholas M. Synovic",
    "Matthew Hyatt",
    "George K. Thiruvathukal",
]

from argparse import HelpFormatter
from operator import attrgetter


class SortingHelpFormatter(HelpFormatter):
    def add_arguments(self, actions):
        actions = sorted(actions, key=attrgetter("option_strings"))
        super(SortingHelpFormatter, self).add_arguments(actions)


def busFactorArgs() -> Namespace:
    parser: ArgumentParser = ArgumentParser(
        prog=f"{name} Bus Factor Calculator",
        description="A tool to calculate the bus factor of a Git repository",
        epilog=f"Author(s): {', '.join(authors)}",
        formatter_class=SortingHelpFormatter,
    )
    parser.add_argument(
        "-i",
        "--input",
        type=str,
        help="Commits JSON file. DEFAULT: ./commits_loc.json",
        default="commits_loc.json",
    )
    parser.add_argument(
        "-b",
        "--bin",
        help="Bin containing the number of days between computed bus factor values. DEFAULT: 1",
        type=int,
        default=1,
    )
    parser.add_argument(
        "-v",
        "--version",
        help="Display version of the tool",
        action="version",
        version=f"{versionName}: {version()} ",
    )
    parser.add_argument(
        "-o",
        "--output",
        help="Output JSON file. DEFAULT: ./bus_factor.json",
        type=str,
        default="bus_factor.json",
    )
    parser.add_argument(
        "-a",
        "--alpha",
        help="The percent change of the code base a developer needs to contribute in a time interval . DEFAULT: 0.8",
        type=float,
        default=0.8,
    )
    return parser.parse_args()


def graphArgs() -> Namespace:
    parser: ArgumentParser = ArgumentParser(
        prog=f"{name} Bus Factor Grapher",
        description="A tool to graph the bus factor of a repository",
        epilog=f"Author(s): {', '.join(authors)}",
        formatter_class=SortingHelpFormatter,
    )
    parser.add_argument(
        "-y",
        "--y-data",
        help="The key to use for graphing the y axis data. DEFAULT: busFactor",
        type=str,
        default="busFactor",
    )
    parser.add_argument(
        "-i",
        "--input",
        help=f"JSON export from clime-git-bus-factor-compute. DEFAULT: ./bus_factor.json",
        type=str,
        required=False,
        default="bus_factor.json",
    )
    parser.add_argument(
        "-o",
        "--output",
        help="Filename of the graph. DEFAULT: ./bus_factor.pdf",
        type=str,
        required=False,
        default="bus_factor.pdf",
    )
    parser.add_argument(
        "--type",
        help="Type of figure to plot. DEFAULT: bar",
        type=str,
        required=False,
        default="bar",
    )
    parser.add_argument(
        "--title",
        help='Title of the figure. DEFAULT: ""',
        type=str,
        required=False,
        default="",
    )
    parser.add_argument(
        "--x-label",
        help='X axis label of the figure. DEFAULT: ""',
        type=str,
        required=False,
        default="",
    )
    parser.add_argument(
        "--y-label",
        help='Y axis label of the figure. DEFAULT: ""',
        type=str,
        required=False,
        default="",
    )
    parser.add_argument(
        "--stylesheet",
        help='Filepath of matplotlib stylesheet to use. DEFAULT: ""',
        type=str,
        required=False,
        default="",
    )
    parser.add_argument(
        "-v",
        "--version",
        help="Display version of the tool",
        action="version",
        version=f"{versionName}: {version()} ",
    )

    return parser.parse_args()



from argparse import Namespace

import pandas
from pandas import DataFrame

from clime_bus_factor.args import busFactorArgs


def buildBusFactor(
    df: DataFrame, *, bin: int, alpha: float = 0.0, stor: str = "busFactor"
) -> DataFrame:
    day_key = "author_days_since_0"
    lastday = df[day_key].max() + bin
    bins = list(range(0, lastday, bin))

    df["commitBin"] = pandas.cut(df[day_key], bins=bins, include_lowest=True)
    bins = df["commitBin"].unique().tolist()

    data = []
    for bin in bins:

        item = {"days_since_0": int(bin.left) if bin.left > 0 else 0}

        if alpha > 0:
            temp = df[df["commitBin"] == bin]
            abs_list = lambda l: [abs(item) for item in l]
            significance = alpha * sum(abs_list(temp["dkloc"].tolist()))

            bf = 0
            authors = set(temp["author_email"].tolist())
            for author in authors:
                author_dloc = sum(
                    abs_list(temp[temp["author_email"] == author]["dkloc"].tolist())
                )
                if author_dloc > significance:
                    bf += 1

            temp = temp[temp["dkloc"] > significance]

            item[stor] = bf
        else:
            item[stor] = len(df[df["commitBin"] == bin]["author_email"].unique())

        data.append(item)

    return DataFrame(data)


def main() -> None:
    args: Namespace = busFactorArgs()

    if args.bin < 1:
        print(f"Bin arguement must be an integer greater than 0: {args.bin}")
        quit(1)
    if args.alpha > 1:
        print("Invalid alpha value. Must be alpha =< 1 and alpha >= 0")
        quit(2)
    if args.alpha < 0:
        print("Invalid alpha value. Must be alpha =< 1 and alpha >= 0")
        quit(3)

    df: DataFrame = pandas.read_json(args.input).T
    bf: DataFrame = buildBusFactor(df, bin=args.bin, alpha=args.alpha, stor="busFactor")
    cd: DataFrame = buildBusFactor(df, bin=args.bin, alpha=0, stor="developerCount")

    cdColumn = cd["developerCount"]
    bf.join(cdColumn).to_json(args.output, indent=4)


if __name__ == "__main__":
    main()
